# Generated by Django 4.2.11 on 2024-09-26 21:34

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("killstats", "0003_attacker_remove_killmail_attackers_and_more"),
    ]

    # Moved migration to command line migration (management/commands/migrate_old_killmails.py)
