"""
Constants
"""

# AA Killstats
from killstats import __title__

REQUESTS_TIMEOUT = (5, 30)
RETRY_DELAY = 10

LAST_REQUEST_KEY = f"{__title__.upper()}_R2Z2_LAST_REQUEST"
RETRY_AFTER_KEY = f"{__title__.upper()}_R2Z2_RETRY_AFTER"
